#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from datetime import datetime

# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui import
from bewype.core import ui
# bewype component import
from bewype.core.component.form_field import FormField
# bewype config import
from bewype.core.config import Config

class DatePicker(FormField):

    def add_static_resources(self):
        FormField.add_static_resources(self, file_=__file__)

    def get_form_field(self, item):
        # get key for date entry
        _id = "%sPickerButton" % self.id_
        _value = self._get_value(item)
        _label = _("Date")
        # return the picker
        return ui.PickerButton(id_=_id, class_=self.class_, label=_label,
                value=_value)

    def _get_value(self, item):
        _value = FormField._get_value(self, item)
        if _value is None:
            return [None, None]
        else:
            return [
                    _value.strftime(Config().date_format()),
                    _value.strftime(Config().date_format(default=True))
                    ]

    def _set_value(self, item, value):
        # get default date format
        _d_format = Config().date_format()
        # get date object
        _d_value = datetime.strptime(value,  _d_format)
        # parent set
        return FormField._set_value(self, item, _d_value)

