#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from datetime import datetime
# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui import
from bewype.core import ui
# bewype component import
from bewype.core.component.form_field import FormField
# bewype config import
from bewype.core.config import Config

class Entry(FormField):

    def _add_component_templates(self, file_, key_dict=None, dir_path=None):
        # get the entry validator type
        _validator = self.get_validator()
        # update the template dict
        if key_dict is None:
            key_dict = {"validator": _validator}
        else:
            key_dict["validator"] = _validator
        # call parent with updated template
        FormField._add_component_templates(self, file_, key_dict=key_dict,
                dir_path=dir_path)

    def add_static_resources(self):
        FormField.add_static_resources(self, file_=__file__)

    def get_validator(self):
        """Returns the entry validator identifier: NotEmpty or Date.

        *Note* - should be overriden by a child `class`.
        """
        return None

    def get_form_field(self, item):
        # get key for date entry
        _id = "%sEntry" % self.id_
        _value = self._get_value(item)
        # prepare entry
        return ui.Entry(id_=_id, class_=self.class_, value=_value)

    def _get_value(self, item):
        _value = FormField._get_value(self, item)
        if _value is None:
            return None
        elif self.get_validator() == "Date":
            return _value.strftime(Config().date_format())
        else:
            return _value

    def _set_value(self, item, value):
        # ensure iso date
        if value is not None \
        and self.get_validator() == "Date":
            # get default date format
            _d_format = Config().date_format()
            try:
                # re-format date str
                value = datetime.strptime(value, _d_format)
            except Exception, e:
                # ERROR
                log.err("[entry] _set_value - e: %s" % e)
                # ERROR
                return "Invalid date format, should be '%s'!" \
                        % Config().js_date_format()
        else:
            pass
        # call parent
        return FormField._set_value(self, item, value)

