#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui import
from bewype.core import ui
# bewype component import
from bewype.core.component.form_field import FormField
# bewype message import
from bewype.core.message import RpcMessage, RpcResult
# bewype resource import
from bewype.core.resource import Path, Resource

class FilePicker(FormField):

    def add_static_resources(self):
        FormField.add_static_resources(self, file_=__file__)

    def get_actions(self):
        return [
                "update",
                "upload"
                ]

    def get_form_field(self, item):
        # get key for date entry
        _id = "%sPickerButton" % self.id_
        _value = self._get_value(item)
        _label = _("File")
        # return the picker
        return ui.PickerButton(id_=_id, class_=self.class_, label=_label,
                value=_value)

    def _set_value(self, item, filename):
        """!!!!Disable in method.. parent .. should only be called manually..
        Other file will be removed on update and upload won't be effective !!!!
        (Sorry for that :s!)
        """
        return True

    def __set_value(self, item, filename):
        # get old filename
        _old_filename = self._get_value(item)
        # remove old resource first
        if self._remove_file(_old_filename):
            # parent set
            return FormField._set_value(self, item, filename)
        else:
            return False

    def _remove_file(self, filename):
        # no file - nothing to do .. it's ok;)
        if filename is None:
            return True
        # do remove
        else:
            # get file path
            _p = Path(path=filename,
                    language_code=self.page.language_code,
                    widget_element=self.widget_element)
            # remove old file
            return Resource(_p.get_resource_path()).remove() \
                and Resource(_p.get_static_path()).remove()

    def _write_file(self, filename, data):
        # get file name
        if filename is None \
        or data is None:
            return False
        else:
            # get file path
            _p = Path(path=filename, language_code=self.page.language_code,
                    widget_element=self.widget_element)
            # set resource file content and return update flag
            return Resource(_p.get_resource_path()).write(data) \
                    and Resource(_p.get_static_path()).write(data)

    def do_upload(self, data, public=False):
        # get avatar id
        _avatarId = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        # not connected
        if _avatarId is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        # is connected
        else:
            # get content
            _m = RpcMessage(data)
            # get item
            _item = self._select(_m.get_int("itemId"))
            # check found
            if _item is None:
                # simple message for users
                _r.error(_("Item not found!"))
                return _r
            else:
                # get item
                _item = self._select(_m.get_int("itemId"))
                # get value key
                _value_key = self.get_value_key()
                # get file name
                _filename = _m.get_val(_value_key)
                _data = _m.get_val("userfile")
                # write file
                if self.__set_value(_item, _filename) \
                and self._write_file(_filename, _data):
                    _r.success(_("File sucessfully uploaded"))
                    _r.add(_value_key, _filename)
                    return _r
                else:
                    _r.error(_("Upload failed"))
                    return _r

