#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui imports
from bewype.core import ui
# bewype component import
from bewype.core.component.component import Component
# bewype db import
from bewype.core.db import Query
# bewype message import
from bewype.core.message import RpcMessage, RpcResult

class FormChildren(Component):

    def __init__(self, orm_cls_name=None, children_cls_name=None,
            label=None):
        # Create parent
        Component.__init__(self, orm_cls_name=orm_cls_name, label=label)
        # specific for children form
        self._ch_q = Query(children_cls_name)

    def get_select_label(self):
        raise NotImplementedError("Please implement this!")

    def get_actions(self):
        return [
                "append",
                "remove",
                "update"
                ]

    def add_static_resources(self, file_=None):
        # call parent
        ui.Widget.add_static_resources(self, file_=__file__)

    def get_model_row(self, row):
        return [
                getattr(row, self.get_value_key()),
                getattr(row, self._ch_q.key_id)
                ]

    def get_combobox_model(self, item):
        # check item
        if item is None:
            return []
        else:
            # prepare criteria list
            _criteria_list = [
                    self._ch_q.key_col!=getattr(item, self._q.key_id),
                    getattr(self._ch_q.obj, "parent_id")==None
                    ]
            # get items from children table
            _items = self._ch_q.get_items(criteria_list=_criteria_list)
        # parse values only
        return map(self.get_model_row, _items)

    def _get_active_value(self, sub_item):
        #
        if hasattr(sub_item, self._ch_q.key_id):
            return getattr(sub_item, self._ch_q.key_id)
        else:
            # ERROR
            log.err("[Form] _get_active_value - %s bad key: %s" \
                    % (sub_item, self._ch_q.key_id))
            return None

    def get_children_model(self, item):
        if item is None:
            return []
        else:
            # get children
            _item_id = getattr(item, self._q.key_id)
            _children_items = self._get_children(_item_id)
        # return model
        return map(self.get_model_row, _children_items)

    def _get_field_box(self, title):
        # init hbox
        _hbox = ui.HBox(id_="%sFormChildren" % self.id_, class_=self.class_)
        # add Label
        _hbox.add(ui.Label(class_=self.class_, text=title))
        # return box
        return _hbox

    def get_combobox(self, item):
        # get model
        _model = self.get_combobox_model(item)
        # init combobox of the cell renderer
        _combo = ui.ComboBox(
                id_="%sCombobox" % self.id_,
                class_=self.class_,
                model=_model
                )
        # create cell text renderer
        _cell = ui.CellRendererText(class_=self.class_)
        # set cellrenderer
        _combo.pack_start(_cell)
        _combo.add_attribute(_cell, "text", 0)
        # do return
        return _combo

    def get_link_add(self):
        # create simple link
        _a_id = "%sAppendButton" % self.id_
        _href = "#%s" % self.class_.lower()
        _text = _("Append")
        return ui.Label(id_=_a_id, class_=self.class_, text=_text, href=_href)

    def get_field(self, item):
        # get field box first
        _hbox = self._get_field_box(self.get_select_label())
        _hbox.add(self.get_combobox(item))
        # add add link to parent
        _hbox.add(self.get_link_add())
        # .. return all
        return _hbox

    def _udpate(self, item_id, child_id, parent_id, position):
        # prepare result
        _r = RpcResult()
        # prepare insert dict
        _i_dict = {
                "parent_id": parent_id,
                self._ch_q.position_key: position,
                }
        # do add
        return self._ch_q.update(child_id, _i_dict)

    def _add_child(self, item_id, child_id):
        # prepare result
        _r = RpcResult()
        # get current children
        _children_items = self._get_children(item_id)
        # get selected child
        _selected_item = self._get_child(child_id)
        if _selected_item is None:
            _r.error(_("Selected item not Found!"))
            return _r
        elif _selected_item in _children_items:
            _r.error(_("Child item already added!"))
            return _r
        else:
            if self._udpate(item_id, child_id, item_id, len(_children_items)):
                _r.add("itemId", item_id)
                _r.success(_("Child successfully added!"))
                return _r
            else:
                _r.error(_("Update failed!"))
                return _r

    def _remove_child(self, item_id, child_id):
        # prepare result
        _r = RpcResult()
        if self._udpate(item_id, child_id, None, 0):
            _r.add("itemId", item_id)
            _r.success(_("Child successfully removed!"))
            return _r
        else:
            _r.error(_("Update failed!"))
            return _r


    def _update_child(self, item_id, sorted_list):
        # prepare result
        _r = RpcResult()
        # get current children
        _children_items = self._get_children(item_id)
        # update positions
        for _c_item in _children_items:
            # get child id
            _child_id = str(getattr(_c_item, self._ch_q.key_id))
            # get child position
            _position = sorted_list.index(_child_id)
            #
            if self._udpate(item_id, _child_id, item_id, _position):
                continue
            else:
                _r.error(_("Update failed!"))
                return _r
        # looks ok
        _r.add("itemId", item_id)
        _r.success(_("Children successfully updated!"))
        return _r

    def _get_child(self, child_id):
        return self._ch_q.get_item(self._ch_q.key_id, child_id)

    def _get_children(self, item_id):
        # prepare criteria list
        _criteria_list = [
                getattr(self._ch_q.obj, "parent_id")==item_id
                ]
        # get items from children table
        return self._ch_q.get_items(criteria_list=_criteria_list,
                order_by_name=self._ch_q.position_key)

    def do_append(self, data, public=False):
        # get avatar id
        _avatar_id = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        if _avatar_id is None \
        and public is False:
            # not connected result
            _r.error(_("Not connected!"))
            return _r
        else:
            # parse message
            _m = RpcMessage(data)
            # get item name
            _item_id = _m.get_int("itemId")
            _child_id = _m.get_int("childId")
            #
            if _item_id is None \
            or _child_id is None:
                _r.error(_("Item not found!"))
                return _r
            else:
                # call layout list update method
                _r = self._add_child(_item_id, _child_id)
                # simply return result
                if _r.get_bool("error"):
                    return _r
                # model list for update
                else:
                    # get item
                    _item = self._select(_item_id)
                    # new info
                    _r.add("comboboxModel", self.get_combobox_model(_item))
                    _r.add("childrenModel", self.get_children_model(_item))
                    # update result
                    return _r

    def do_remove(self, data, public=False):
        # get avatar id
        _avatar_id = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        if _avatar_id is None \
        and public is False:
            # not connected result
            _r.error(_("Not connected!"))
            return _r
        else:
            # parse message
            _m = RpcMessage(data)
            # get item name
            _item_id = _m.get_int("itemId")
            _child_id = _m.get_int("childId")
            #
            if _item_id is None or _child_id is None:
                _r.error(_("Item not found!"))
                return _r
            else:
                # call layout list update method
                _r = self._remove_child(_item_id, _child_id)
                # simply return result
                if _r.get_bool("error"):
                    return _r
                # model list for update
                else:
                    # get item
                    _item = self._select(_item_id)
                    # new info
                    _r.add("comboboxModel", self.get_combobox_model(_item))
                    _r.add("childrenModel", self.get_children_model(_item))
                    # update result
                    return _r

    def do_update(self, data, public=False):
        # get avatar id
        _avatar_id = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        if _avatar_id is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        else:
            # parse message
            _m = RpcMessage(data)
            # get item name
            _item_id = _m.get_int("itemId")
            _sorted_list = _m.get_list("sortedList")
            #
            if _item_id is None:
                _r.error(_("Item not found!"))
                return _r
            elif _sorted_list is None:
                _r.error(_("Nothing to update!"))
                return _r
            else:
                # do udpate
                return self._update_child(_item_id, _sorted_list)

    def do_reload(self, data, public=False):
        # get avatar id
        _avatar_id = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        if _avatar_id is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        else:
            # parse message
            _m = RpcMessage(data)
            # get item name
            _item_id = _m.get_int("itemId")
            # empty form
            if _item_id is None:
                _r.success(_("OK"))
                _r.add("comboboxModel", [])
                _r.add("childrenModel", [])
                return _r
            # return models
            else:
                _item = self._select(_item_id)
                _r.success(_("OK"))
                _r.add("comboboxModel", self.get_combobox_model(_item))
                _r.add("childrenModel", self.get_children_model(_item))
                return _r

