#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui import
from bewype.core import ui
# bewype component import
from bewype.core.component.component import Component

class FormField(Component):
    """Form oriented component for form widgets like:

        - entry
        - date picker
        - text editor
        - Etc.

    Offers common methods for form component among already existing base
    methods:

        - `get_actions`
        - `get_field`

    """

    def __init__(self, widget_name=None, orm_cls_name=None, label=None,
            horizontal=True):
        # init parent
        Component.__init__(self, widget_name=widget_name, orm_cls_name=orm_cls_name,
                label=label)
        # keep orientation flag
        self.horizontal = horizontal

    def add_static_resources(self, file_=None):
        Component.add_static_resources(self, file_=file_)

    def get_actions(self):
        return [
                "cancel",
                "update"
                ]

    def _get_field_box(self):
        """Adds a label before the form widget.
        """
        # init box
        if self.horizontal:
            _box = ui.HBox(class_=self.class_)
        else:
            _box = ui.VBox(class_=self.class_)
        # add Label
        _box.add(ui.Label(class_=self.class_, text=self._label))
        # return box
        return _box

    def get_form_field(self, item):
        """Returns the form field: `Entry`, `Picker`, `TextView`, Etc.

        *Note - Should be overriden by a child `class`.*

        :params item: db item to be edited in the form field. If None an empty
                      field should be returned
        """
        raise NotImplementedError("Please implement this!")

    def get_field(self, item):
        """Returns the component field to render.

        :params item: db item to be edited in the form field. If None an empty
                      field should be returned
        """
        _form_field = self.get_form_field(item)
        # if no label return form_field directly
        if self._label is None:
            return _form_field
        else:
            _hbox = self._get_field_box()
            _hbox.add(_form_field)
            return _hbox

