#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# twisted import
from twisted.python import log

# nevow import
from nevow.flat import flatten
from nevow.i18n import _

# bewype ui imports
from bewype.core import ui
# bewype component import
from bewype.core.component.component import  Component
# bewype db import
from bewype.core.db import Query

class Menu(Component):

    def __init__(self, orm_cls_name=None, children_cls_name=None,
            label=None):
        # Create parent
        Component.__init__(self, orm_cls_name=orm_cls_name, label=label)
        # specific for children form
        self._ch_q = Query(children_cls_name)

    def get_css_dict(self):
        return dict()

    def _add_component_templates(self, file_, key_dict=None, dir_path=None):
        # call parent with updated template
        Component._add_component_templates(self, file_,
                key_dict=self.get_css_dict(), dir_path=dir_path)

    def get_href_value(self, menu_element):
        return "#%s" % self.id_.lower()

    def get_src_value(self, menu_element):
        return None

    def get_actions(self):
        return []

    def _add_menu_item(self, menu, menu_element):
        # prepare item params
        _text = getattr(menu_element, self.get_value_key())
        # get the menu item link
        _href = self.get_href_value(menu_element)
        # get the menu item web path (for img)
        _src = self.get_src_value(menu_element)
        # init item
        _menu_item = ui.MenuItem(class_=self.class_, text=_text, href=_href,
                src=_src)
        # get children
        _children = self._get_children(menu_element)
        # add sub menu item to the current item
        map(partial(self._add_menu_item, _menu_item), _children)
        # add menu element to the menu
        menu.add(_menu_item)

    def get_field(self, item):
        # init menu
        _id = "%sMenu" % self.id_
        _menu = ui.Menu(id_=_id, class_=self.class_)
        # get children
        _children = self._get_children(item)
        if item is None:
            pass
        else:
            # add items
            map(partial(self._add_menu_item, _menu), _children)
        # return menu at the end
        return _menu

    def _get_value(self, item):
        # get ui obj first
        _menu = self.get_field(item)
        # get corresponding doc factory
        _doc = _menu.get_docFactory()
        # return serialized doc
        return flatten(_doc)

    def _get_children(self, item):
        if item is None:
            return []
        else:
            # prepare criteria list
            _criteria_list = [
                    getattr(self._ch_q.obj, "parent_id")==\
                            getattr(item, self._q.key_id)
                    ]
            # get items from children table
            return self._ch_q.get_items(criteria_list=_criteria_list,
                    order_by_name=self._ch_q.position_key)

