#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# sqlalchemy import
from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import sessionmaker, mapper, clear_mappers

# bewype config import
from bewype.core.config import Config

class SessionManager(object):
    """Session manager for db connection simple connection
    """

    class __Singleton:
        """Manager object for the singleton
        """

        def __init__(self, echo=False):
            """Connect the first time and keep the session.

            If config is None we create an SQLite in memory connection. Should
            be used with unit testing.
            """
            # get db connection params
            self.url = Config().db_url()
            # connect for prod
            self.engine = create_engine(self.url, echo=echo)
            try:
                #
                self.metadata = MetaData()
                # create session
                self.sessionmaker = sessionmaker()
                self.sessionmaker.configure(bind=self.engine)
                # create the main session of the application
                self.session = self.sessionmaker()
                # bind database
                self.metadata.reflect(bind=self.engine)
            except Exception, e:
                # ERROR
                log.err("[__Singleton] __init__ - e: %s" % e)
                # set engin to None
                self.engine = None
                self.sessionmaker = None
                self.session = None
                self.metadata = None
                #
                return

        def get_table(self, table_name):
            """
            """
            if self.metadata is None:
                return None
            elif self.metadata.tables.has_key(table_name):
                return self.metadata.tables[table_name]
            else:
                return None

        def get_tables(self):
            """
            """
            if self.metadata is None:
                return dict()
            else:
                return self.metadata.tables

    # session manager instance
    instance = None

    def __new__(c, echo=False, force=False):
        """Singleton new init.
        """
        # split clear all procedure in an inner method
        def _clear_all():
            """
            """
            if SessionManager.instance is None:
                pass
            else:
                # close session
                SessionManager.instance.session.close()
                # clear mapping for next use
                clear_mappers()
                # del instance
                del SessionManager.instance
        # split get new method procedure in an inner method
        def get_new_instance():
            """
            """
            # create a new instance
            _instance = SessionManager.__Singleton(
                            echo=echo
                            )
            if _instance.engine is None:
                # empty instance just in case
                SessionManager.instance = None
            else:
                # update instance
                SessionManager.instance = _instance
            # return instance
            return SessionManager.instance
        # if doesn't already initialized
        if SessionManager.instance is None or force is True:
            # first clear
            _clear_all()
            # update and return instance
            return get_new_instance()
        else:
            # return the manager object
            return SessionManager.instance
