#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyton import
import os

# twisted import
from twisted.python import log

# turbomail import
from turbomail import Message
from turbomail.control import interface

# bewype config import
from bewype.core.config import Config

class Sender(object):
    """Simple email sender based on turbomail library. Get setting from main
    config, manage txt and html messages with attachments.

    :see: `turbomail`
    """

    def __init__(self, email_from):
        """Creates new sender instance with the email from value. smtp server
        address and debug mode come from the application config.

        :param email_from: str value of the from email address to use, ex.:
                           admin@bewype.org
        """
        # set email from
        self.email_from = email_from
        # get the email config section
        _section_email = Config().get_section("email")
        #
        if _section_email is None:
            pass
        else:
            # get smtp server
            _mail_smtp_server = _section_email["mail_smtp_server"]
            # get transport
            _mail_transport = _section_email["mail_transport"]
            # activate email framework
            _config_init = {
                    "mail.on": True,
                    "mail.smtp.server": _mail_smtp_server,
                    "mail.transport": _mail_transport,
                    }
            # start turbo mail engine
            interface.start(_config_init)

    def _attach_file(self, msg, file_path):
        """Attach a given file to the current message.

        :param msg: message that recieve the attachement
        :param file_path: resource path of the file to be attached

        :see: `turbomail`
        """
        # check path exists
        if os.path.exists(file_path):
            # get base name for the message value
            _file_name = os.path.basename(file_path)
            # get file obj
            _f = file(file_path)
            # do attach
            msg.attach(_f, _file_name)
            # close opened file
            _f.close()
        # do nothing
        else:
            pass

    def send(self, email_to, email_subject, content_txt,
            content_html=None, attachment_list=None):
        """Send an email using the main email from address and given parameters.
        Email to address, email subject and txt content are mandatory.

        Html content and attachments ar optional.

        :param email_to: email to address
        :param email_subject: email subject to be sent
        :param content_txt: text basic content to be sent
        :param content_html: html rich content to be sent (optional)
        :param attachment_list: list of resource paths to be attached (optional)

        :see: `turbomail`
        """
        # not set do nothing
        if self.email_from is None \
        or email_to is None \
        or email_subject is None \
        or content_txt is None:
            return False
        else:
            # prepare message
            _msg = Message(self.email_from, email_to, email_subject)
            # set plain txt
            _msg.plain = content_txt
            # manage html content and attached files
            if content_html is None:
                pass
            else:
                _msg.rich = content_html
            # little check
            if attachment_list is None:
                pass
            else:
                # add attachements
                for _f_path in attachment_list:
                    self._attach_file(_msg, _f_path)
            # do send
            _msg.send()
            # sent!
            return True

