#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from datetime import datetime

# twisted import
from twisted.python import log

# nevow import
from nevow.flat import serialize
from nevow.i18n import PlaceHolder

# bewype config import
from bewype.core.config import Config

def _serialize_list(d_list, context):
    """
    """
    # serialize i18n values
    for _idx, _v in enumerate(d_list):
        if isinstance(_v, dict):
            d_list[_idx] = _serialize_dict(_v, context)
        if isinstance(_v, list):
            d_list[_idx] = _serialize_list(_v, context)
        elif isinstance(_v, PlaceHolder):
            d_list[_idx] = serialize(_v, context)
        else:
            continue
    # return the serialized list
    return d_list

def _serialize_dict(d_dict, context):
    """
    """
    # get default date format
    _default_py_date_format = Config().date_format()
    # serialize i18n values
    for _k, _v in d_dict.items():
        if isinstance(_v, dict):
            d_dict[_k] = _serialize_dict(_v, context)
        if isinstance(_v, list):
            d_dict[_k] = _serialize_list(_v, context)
        elif isinstance(_v, datetime):
            d_dict[_k] = _v.strftime(_default_py_date_format)
        elif isinstance(_v, PlaceHolder):
            d_dict[_k] = serialize(_v, context)
        else:
            continue
    # return the serialized dict
    return d_dict

class RpcResult(object):
    """
    """

    def __init__(self, result=None):
        """
        """
        if result is None:
            self.result = {}
        else:
            self.result = result

    def has_key(self, key):
        """
        """
        return self.result.has_key(key)

    def get_val(self, key):
        """
        """
        if self.result.has_key(key):
            return self.result[key]
        else:
            return None

    def get_int(self, key):
        """
        """
        # get str val first
        _val = self.get_val(key)
        try:
            return int(_val)
        except Exception, e:
            # ERROR
            log.err("[RpcResult] get_int - %s|%s e: %s" \
                    % (self.result, key, e))
            # return str by default
            return _val

    def get_bool(self, key):
        """
        """
        # get str val first
        _val = self.get_val(key)
        return _val == "True" or _val == "true" or _val == "1"

    def get_list(self, key):
        """
        """
        pass

    def add(self, key, value):
        """
        """
        self.result[key] = value

    def clear(self):
        """
        """
        self.result = {}

    def remove(self, key):
        """
        """
        del self.result[key]

    def error(self, message):
        """
        """
        self.result["error"] = True
        self.set_message(message)

    def success(self, message):
        """
        """
        self.result["error"] = False
        self.set_message(message)

    def set_message(self, message):
        """
        """
        self.result["message"] = message

    def serialize(self, context):
        """
        """
        return _serialize_dict(self.result, context)

