#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os
#
from datetime import datetime
from ConfigParser import SafeConfigParser

# sqlalchemy import
from sqlalchemy import Column, Table
from sqlalchemy.orm import  mapper
from sqlalchemy.types import DateTime, Integer, Unicode

# load versions
import migrate.versioning.api
from migrate.versioning.exceptions import DatabaseAlreadyControlledError

# bewype config import
from bewype.core.config import Config
# bewype db import
from bewype.core.db import model, SessionManager

class _DbManager(object):
    """
    """

    class __Singleton:
        """
        """

        def __init__(self):
            """
            """
            # remove old db file
            if os.path.exists("bewype_test.db"):
                # remove the db file at the end
                os.remove("bewype_test.db")
            else:
                pass
            # do upgrade
            _url = "sqlite:///bewype_test.db"
            _repository_path = Config().db_repository()
            try:
                migrate.versioning.api.version_control(_url, _repository_path)
                migrate.versioning.api.upgrade(_url, _repository_path)
            except DatabaseAlreadyControlledError:
                # it's ok
                pass

            # create config object
            _config = SafeConfigParser()
            # set minimal value
            _config.add_section("db")
            _config.set("db", "url", _url)
            _config.set("db", "echo", "False")
            Config(config=_config, force=True)

            # init the db connection singleton
            _session_manager = SessionManager()
            # do init mapping for orm test
            model.init_mapping()
            # create the tables in the in-memory db
            _session_manager.metadata.create_all(_session_manager.engine)

    # singleton instance
    instance = None

    def __new__(c):
        """Singleton new init.
        """
        # if doesn't already initialized
        if not _DbManager.instance:
            # create a new instance
            _DbManager.instance = _DbManager.__Singleton()
        # return the manager object
        return _DbManager.instance

