#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# nevow import
from nevow import tags as T

# bewype import
from bewype.core.ui.container import Container

# Box
class BoxItem(object):

    def __init__(self, widget, expand=True, fill=True, padding=0):
        # current widget
        self.widget = widget
        # style settings
        self.expand = expand
        self.fill = fill
        self.padding = padding

    def set_page(self, page):
        if hasattr(self.widget, "set_page"):
            return self.widget.set_page(page)
        else:
            pass

    def get_callbacks(self):
        if hasattr(self.widget, "get_callbacks"):
            return self.widget.get_callbacks()
        else:
            pass

    def _update_callbacks(self):
        if hasattr(self.widget, "_update_callbacks"):
            self.widget._update_callbacks()
        else:
            pass

class Box(Container):

    def __init__(self, id_=None, class_=None, homogeneous=True, spacing=0):
        # init parent
        Container.__init__(self, id_=id_, class_=class_)
        # set attributes
        self.homogeneous = homogeneous
        self.spacing = spacing

    def pack_start(self, widget, expand=True, fill=True, padding=0):
        # set box item
        _box_item = BoxItem(widget, expand=expand, fill=fill, padding=padding)
        # add item to the container
        self.widgets.insert(0, _box_item)

    def pack_end(self, widget, expand=True, fill=True, padding=0):
        # set box item
        _box_item = BoxItem(widget, expand=expand, fill=fill, padding=padding)
        # add item to the container
        self.widgets.append(_box_item)

    def reorder_child(self, widget, position):
        # get and remove the box item from the current list
        for _box_item in self.widgets:
            if _box_item.widget is widget:
                self.widgets.remove(_box_item)
                break
            else:
                pass
        # put the item to the container at the expected position
        self.widgets.insert(position, _box_item)

    def set_homogeneous(self, homogeneous):
        self.homogeneous = homogeneous

    def get_homogeneous(self):
        return self.homogeneous

    def set_spacing(self, spacing):
        self.spacing = spacing

    def get_spacing(self):
        return spacing

def update_hbox_td_list(class_, width, box_item):
    #
    if width is None and not box_item.expand:
        _td = T.td(class_=class_)
    else:
        _td = T.td(class_=class_, width=width)
    # init style str
    _style = "padding: %s;" % box_item.padding
    # update style fill
    if box_item.fill:
        _style += " width: 100%;"
    else:
        pass
    # set div for element
    _div = T.div(class_=class_, style=_style)
    #
    if hasattr(box_item.widget, "get_docFactory"):
        _element = box_item.widget.get_docFactory()
    else:
        _element = box_item.widget
    #
    return _td[_div[_element]]

# HBox
class HBox(Box):

    def __init__(self, id_=None, class_=None, homogeneous=True, spacing=0):
        # init parent
        Box.__init__(self, id_=id_, class_=class_,
                homogeneous=homogeneous, spacing=spacing)

    def get_docFactory(self):
        _table = T.table(id=self.id_, class_=self.class_,
                cellspacing=self.spacing)
        # prepare homogenous style value
        if self.homogeneous == True \
        and len(self.widgets) != 0:
            _width = "%s%%" % str(100 / len(self.widgets))
        else:
            _width = None
        # little check
        if len(self.widgets) == 0:
            return _table
        else:
            # prepare get columns func
            _update_func = partial(
                    update_hbox_td_list, self.class_, _width)
            # render row
            return _table[
                    T.tr[map(_update_func, self.widgets)]
                    ]

def update_vbox_tr_list(class_, box_item):
    # init style str
    _style = "padding: %s;" % box_item.padding
    # update style fill
    if box_item.fill:
        _style += " width: 100%;"
    else:
        pass
    # set div for element
    _div = T.div(class_=class_, style=_style)
    # get box element content
    _element = box_item.widget.get_docFactory()
    # return prepared row
    return T.tr[T.td(class_=class_)[_div[_element]]]

class VBox(Box):

    def __init__(self, id_=None, class_=None, homogeneous=True, spacing=0):
        # init parent
        Box.__init__(self, id_=id_, class_=class_,
                homogeneous=homogeneous, spacing=spacing)

    def get_docFactory(self):
        # prepare table tag
        _table = T.table(id=self.id_, class_=self.class_,
                cellspacing=self.spacing)
        # little check
        if len(self.widgets) == 0:
            return _table
        else:
            # prepare getter row func
            _update_func = partial(update_vbox_tr_list, self.class_)
            # return prepared table
            return _table[
                    map(_update_func, self.widgets)
                    ]

