#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# twisted import
from twisted.python import log

# nevow import
from nevow import tags as T

# bewype import
from bewype.core.ui.box import Box, HBox, VBox
from bewype.core.ui.button import Button, CheckButton, RadioButton

# HButtonBox
class HButtonBox(HBox):

    def __init__(self, id_=None, class_=None):
        """
        """
        # init parent
        HBox.__init__(self, id_=id_, class_=class_)

# VButtonBox
class VButtonBox(VBox):

    def __init__(self, id_=None, class_=None):
        """
        """
        # init parent
        VBox.__init__(self, id_=id_, class_=class_)

# ButtonBox
class ButtonBox(HButtonBox, VButtonBox):
    """
    """

    def __init__(self, id_=None, class_=None, model=None, active_list=None,
            horizontal=False, button_cls=None):
        """
        """
        #
        self.horizontal = horizontal
        # ButtonBox
        if self.horizontal is True:
            HButtonBox.__init__(self, id_=id_, class_=class_)
        else:
            VButtonBox.__init__(self, id_=id_, class_=class_)
        # set values
        self.model = model
        # ensure default active list
        if active_list is None:
            self.active_list = []
        else:
            self.active_list = active_list
        # set button class
        self.button_cls = button_cls

    def _get_row(self, button_group, model_row):
        """
        """
        model_row["class_"] = self.class_
        # Button factory
        # default
        if self.button_cls is None:
            _button = Button(**model_row)
        # CheckBox
        elif self.button_cls == CheckButton:
            _button = CheckButton(**model_row)
        # RadioBox
        elif self.button_cls == RadioButton:
            _button = RadioButton(**model_row)
            _button.set_group(button_group)
        else:
            return
        # set activity
        if model_row.has_key("value") \
        and hasattr(_button, "set_active"):
            _button.set_active(model_row["value"] in self.active_list)
        else:
            pass
        # add button to the list at the end
        self.add(_button)

    def get_docFactory(self):
        """
        """
        # little check
        if self.model is None:
            pass
        else:
            _button_group = list()
            map(partial(self._get_row, _button_group), self.model)
        # ButtonBox
        if self.horizontal is True:
            return HButtonBox.get_docFactory(self)
        else:
            return VButtonBox.get_docFactory(self)

    def set_model(self, model):
        """
        """
        self.model = model

    def get_model(self):
        """
        """
        return self.model

    def set_active_list(self, active_list):
        """
        """
        self.active_list = active_list

    def get_active_list(self):
        """
        """
        return self.active_list

    def get_active_list(self, active_list):
        """
        """
        self.active_list = active_list

# CheckBox
class CheckBox(ButtonBox):
    """
    """

    def __init__(self, id_=None, class_=None, model=None, active_list=None,
            horizontal=False):
        """
        """
        # ButtonBox
        ButtonBox.__init__(self, id_=id_, model=model, active_list=active_list,
                horizontal=horizontal, button_cls=CheckButton)

# RadioBox
class RadioBox(ButtonBox):
    """
    """

    def __init__(self, id_=None, class_=None, model=None, active_list=None,
            horizontal=False):
        """
        """
        # ButtonBox
        ButtonBox.__init__(self, id_=id_, model=model, active_list=active_list,
                horizontal=horizontal, button_cls=RadioButton)

