#!/usr/bin/env python
# -*- coding: utf-8 -*-

# bewype import
from bewype.core.ui.combo_box import ComboBox
from bewype.core.ui.container import Container
from bewype.core.ui.entry import Entry
from bewype.core.ui.label import Label

class CellRenderer(Container):

    def __init__(self, id_=None, class_=None):
        Container.__init__(self, id_=id_, class_=class_)

class CellRendererText(CellRenderer):
    """Object that renders an `Entry` or a `Label` into a `TreeView` cell.
    """

    def __init__(self, id_=None, class_=None):
        CellRenderer.__init__(self, id_=id_, class_=class_)
        # current attributes
        self.editable = None
        #
        self.markup = None
        self.text = None
        self.href = None
        self.name = None

    def get_docFactory(self):
        # FOR INPUT
        if self.editable is True:
            _widget = Entry(
                    class_=self.class_,
                    value=self.text,
                    markup=self.markup
                    )
        # FOR TEXT
        else:
            _widget = Label(
                    class_=self.class_,
                    text=self.text,
                    href=self.href,
                    markup=self.markup,
                    name=self.name
                    )
        # return the expected factory at the end
        return _widget.get_docFactory()

    def set_text(self, text):
        self.text = text

    def get_text(self):
        return self.text

    def set_markup(self, markup):
        self.markup = markup

    def get_markup(self):
        return self.markup

    def set_href(self, href):
        self.href = href

    def get_href(self):
        return self.href

    def set_name(self, name):
        self.name = name

    def get_name(self):
        return self.name

class CellRendererCombo(CellRenderer):
    """Object that renders a `ComboBox` into a `TreeView` cell.
    """

    def __init__(self, id_=None, class_=None, model=None, text_column=0):
        CellRenderer.__init__(self, id_=id_, class_=class_)
        # init combobox of the cell renderer
        self.combo = ComboBox(class_=self.class_, model=model)
        #
        _cell = CellRendererText(class_=class_)
        #
        self.combo.pack_start(_cell)
        self.combo.add_attribute(_cell, "text", text_column)
        # rendering attribute
        self.active_list = None
        self.multiple = None

    def get_docFactory(self):
        # update content
        self.combo.set_active_list(self.active_list)
        self.combo.set_multiple(self.multiple)
        #
        return self.combo.get_docFactory()

    def set_active_list(self, active_list):
        self.active_list = active_list

    def get_active_list(self):
        return self.active_list

    def set_multiple(self, multiple):
        self.multiple = multiple

    def get_multiple(self):
        return self.multiple

