#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# twisted import
from twisted.python import log

# nevow import
from nevow import tags as T

# bewype import
from bewype.core.ui.container import Container
from bewype.core.ui.image import Image
from bewype.core.ui.label import Label

def _get_elementFactory(class_, element):
    # prepare ul item
    if hasattr(element, "get_docFactory"):
        # render element title
        _title = element.get_docFactory()
    else:
        # None
        _title = None
    # render sub list
    if element.widgets is None \
    or len(element.widgets) == 0:
        _sub_list = None
    else:
        _sub_list = list()
        # udpate list without None values
        for _w in element.widgets:
            _sub_element = _get_elementFactory(class_, _w)
            # should not be None
            if _sub_element is None:
                continue
            else:
                _sub_list.append(_sub_element)
    # ensure good sub list factory
    if _title is None \
    and _sub_list is None:
        return None
    elif _title is None:
        return T.li(class_=class_)[
                T.ul()[_sub_list]
                ]
    elif _sub_list is None:
        return T.li(class_=class_)[
                _title
                ]
    else:
        return T.li(class_="%sDir" % class_)[
                _title,
                T.ul(class_=class_)[_sub_list]
                ]

class MenuItem(Container):

    def __init__(self, id_=None, class_=None, text=None, href=None, src=None,
            model=None):
        # contains the children items
        Container.__init__(self, id_=id_, class_=class_)
        # show a picture
        self.image = Image(class_=class_, src=src)
        # show a text
        self.label = Label(class_=class_, text=text, href=href)
        # set model
        if model is None:
            self.model = []
        else:
            self.model = model
            self._update_children()

    def get_docFactory(self):
        # prepare div
        _span = T.span(id=self.id_, class_=self.class_)
        # get factories
        _imageFactory = self.image.get_docFactory()
        _labelFactory = self.label.get_docFactory()
        # ensure good factory result
        if self.image.src is None \
        and self.label.text is None:
            return _span
        # No image
        elif self.image.src is None:
            return _span[_labelFactory]
        # No text
        elif self.label.text is None:
            return _span[_imageFactory]
        # full factory
        else:
            return _span[_imageFactory, _labelFactory]

    def _update_children(self):
        self.widgets = map(lambda _i: MenuItem(**_i), self.model)

    def set_model(self, model):
        self.model = model
        self._update_children()

    def get_model(self):
        return self.model

class Menu(Container):

    def __init__(self, id_=None, class_=None, model=None):
        # init container
        Container.__init__(self, id_=id_, class_=class_)
        # set model
        self.model = model

    def get_docFactory(self):
        _div = T.div(id=self.id_, class_=self.class_)
        _list = T.ul(id="%sMenu" % self.id_, class_=self.class_)
        # prepare menu element rendering
        _update_func = partial(_get_elementFactory, self.class_)
        # check has widgets
        if len(self.widgets) > 0:
            # rendering and return
            return _div[_list[map(_update_func, self.widgets)]]
        # empty list
        elif self.model is None:
            # return 
            return _div[_list]
        # prepare items from model and return
        else:
            # create items from model
            _items = map(lambda _i: MenuItem(**_i), self.model)
            # rendering and return
            return _div[_list[map(_update_func, _items)]]

    def set_model(self, model):
        self.model = model

    def get_model(self):
        return self.model

