#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from random import randrange

def get_resource_types():
    """Returns the list of the resource types identifers of the application.

    :return: resource identifers list
    :rtype: `list`
    """
    return [
            "css",
            "i18n",
            "img",
            "tmpl",
            "script"
            ]

def generate_key(key_length):
    """Generate unique key of an expected length.

    :param key_length: length of the key to be generated
    :return: a unique key
    :rtype: `str`
    """
    # prepare chr dict
    _letter = map(lambda _i: chr(_i), range(65, 91))
    _letter.extend(map(lambda _i: chr(_i), range(97, 123)))
    #
    return "".join(
            map(lambda _i: _letter[randrange(50)], range(key_length))
            )

def get_id_in_letters(item_id):
    """Returns a number in corresponding number name::

        >>> get_id_in_letters(12)
        'OneTwo'

        >>> get_id_in_letters(49)
        'FourNine'

    :param item_id: number to convert in letter, like a db item id to use in
                    variable name, Etc.
    :return: the correspondant number in letter
    :rtype: `str`
    """
    # our number in letters base
    _letter_list = [
            "Zero",
            "One",
            "Two",
            "Three",
            "Four",
            "Five",
            "Six",
            "Seven",
            "Height",
            "Nine",
            ]
    # the magic join
    return "".join(map(lambda _c: _letter_list[int(_c)], str(item_id)))

def get_class_name(key_name, pattern="_"):
    """Returns a class name like from a method name like or namespace like word.
    By default it uses the `_` character as separator, but it can be overriden::

        >>> get_class_name("test_name")
        'TestName'

        >>> get_class_name("test.name", pattern=".")
        'TestName'

    :param key_name: method name like or namespace like word, ex.: "test_name", Etc.
    :return: the correspondant class name like word, ex.: "TestName"
    :rtype: `str`
    """
    if key_name is None:
        return None
    else:
        # split the name in lower case first
        _splitted_name = key_name.lower().split(pattern)
        # return joined result
        return "".join(map(lambda _n: _n.capitalize(), _splitted_name))

def get_key_name(class_name, pattern="_"):
    """Returns a method name like or namespace like from a class name like word.
    By default it uses the `_` character as separator, but it can be overriden::

        >>> get_key_name("TestName")
        'test_name'

        >>> get_key_name("TestName", pattern=".")
        'test.name'

    :param class_name: class name like word, ex.: "TestName", Etc.
    :return: the correspondant method name like or namespace like word,
             ex.: "test_name"
    :rtype: `str`
    """
    # get letter to split and replace
    _r_letters = map(
            lambda _i: [chr(_i), "%s%s" % (pattern, chr(_i + 32))],
            range(65, 91)
            )
    # set value to update
    _key_name = class_name
    # do update
    for oldL, newl in _r_letters:
        _key_name = _key_name.replace(oldL, newl)
    # and return
    return _key_name[1:]

