
/**
 *
 */
BewypeJS.Ui.BoxParser =
{
    parse: function (boxType, elementId)
    {
        var _params = BewypeJS.Ui.WidgetParser._parse(elementId);
        
        if(MochiKit.Base.isUndefinedOrNull(_params))
        {
            return null;
        }
        else
        {
            if(boxType == "horizontal")
            {
                // return new hbox
                return new BewypeJS.Ui.HBox(_params);
            }
            else
            {
                // return new vbox
                return new BewypeJS.Ui.VBox(_params);
            }
        }
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.WidgetParser.swap(elementId, widget);
    }
};

/**
 *
 */
BewypeJS.Ui.BoxItem = new Class(
{
    initialize: function (widget, params)
    {
        // current widget
        this.widget = widget;
        // style settings
        if (MochiKit.Base.isUndefinedOrNull(params))
        {
            this.expand = true;
            this.fill = true;
            this.padding = null;
        }
        else
        {
            this.expand = BewypeJS.Utils.getVal(params, "expand");
            this.fill = BewypeJS.Utils.getVal(params, "fill");
            this.padding = BewypeJS.Utils.getVal(params, "padding");
        }
    }
});

/**
 *
 */
BewypeJS.Ui.Box = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Container,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);

        //
        this.homogeneous = BewypeJS.Utils.getVal(params, "homogeneous");
        // ensure default value
        if (MochiKit.Base.isUndefinedOrNull(this.homogeneous))
        {
            this.homogeneous = true;
        }
        else
        {
            MochiKit.Base.noop();
        }

        //
        this.spacing = BewypeJS.Utils.getVal(params, "spacing");
        // ensure default value
        if (MochiKit.Base.isUndefinedOrNull(this.spacing))
        {
            this.spacing = "0px";
        }
        else
        {
            MochiKit.Base.noop();
        }
    },

    /**
     * params
     * ------
     * expand = True
     * fill = True
     * padding = 0
     */
    packStart: function (widget, params)
    {
        var boxItem = new BewypeJS.Ui.BoxItem(widget, params);
        // add item to the container
        BewypeJS.Utils.Array.insertToArray(this.widgets, boxItem, 0);
    },

    /**
     * params
     * ------
     * expand = True
     * fill = True
     * padding = 0
     */
    packEnd: function (widget, params)
    {
        var boxItem = new BewypeJS.Ui.BoxItem(widget, params);
        // add item to the container
        this.widgets.push(boxItem);
    },

    add: function (widget)
    {
        this.packEnd(widget)
    },

    setHomogeneous: function (homogeneous)
    {
        this.homogeneous = homogeneous;
    },

    getHomogeneous: function ()
    {
        return this.homogeneous;
    },

    setSpacing: function (spacing)
    {
        this.spacing = spacing;
    },

    getSpacing: function ()
    {
        return this.spacing;
    },

    reorderChild: function (child, position)
    {
         // pass
    }
});

/**
 *
 */
BewypeJS.Ui.HBoxParser =
{
    parse: function (elementId)
    {
        // use parent parser
        return BewypeJS.Ui.BoxParser.parse("horizontal", elementId);
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.BoxParser.swap(elementId, widget);
    }
};

/**
 *
 */
function updateHBoxTDList(classBox, width, boxItem)
{
    //
    var _attrs = {"class": classBox};
    //
    if (MochiKit.Base.isUndefinedOrNull(width) 
        && boxItem.expand == false)
    {
        MochiKit.Base.noop();
    }
    else
    {
        _attrs["width"] = width;
    }

    // init
    var styleStr = "";

    // init style str
    if (MochiKit.Base.isUndefinedOrNull(boxItem.padding))
    {
        // do nothing
        MochiKit.Base.noop();
    }
    else
    {
        styleStr += "padding: %s;" % boxItem.padding;
    }

    // update style fill
    if (!MochiKit.Base.isUndefinedOrNull(boxItem.fill)
        && boxItem.fill == true)
    {
        styleStr += " width: 100%;";
    }
    else
    {
        MochiKit.Base.noop();
    }

    //
    var element = null;
    //
    if (boxItem.widget instanceof BewypeJS.Ui.Box)
    {
        var params = {"tableWidth": "100%"};
        element = boxItem.widget.getDocFactory(params);
    }
    else
    {
        element = boxItem.widget.getDocFactory();
    }

    // set div for element
    var divElement = MochiKit.DOM.DIV(
                {
                    "class": classBox,
                    "style": styleStr
                },
                element
            );
    //
    return MochiKit.DOM.TD(_attrs, divElement);
};

/**
 *
 */
BewypeJS.Ui.HBox = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Box,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);
    },

    getDocFactory: function (params)
    {
        // update attrs
        this.attrs["cellspacing"] = this.spacing;

        //
        if (MochiKit.Base.isUndefinedOrNull(params))
        {
            MochiKit.Base.noop();
        }
        else
        {
            //
            var tableWidth = BewypeJS.Utils.getVal(params, "tableWidth");
            //
            if (MochiKit.Base.isUndefinedOrNull(tableWidth))
            {
                MochiKit.Base.noop();
            }
            else
            {                
                this.attrs["width"] = tableWidth;
            }
        }

        //
        var width = null;
        //
        if (this.homogeneous == true)
        {
            width = (100 / this.widgets.length) + "%";
        }
        else
        {
            width = null;
        }

        //
        if (this.widgets.length > 0)
        {
            // get current class
            var _class = BewypeJS.Utils.getVal(this.attrs, "class");
            // set update func
            var updateFunc = MochiKit.Base.partial(
                    updateHBoxTDList, _class, width);
            //
            var hBoxTDList = MochiKit.Base.map(updateFunc, this.widgets);
            //
            var hBoxTR = TR(null, hBoxTDList);
            //
            var tableBody =  MochiKit.DOM.TBODY(null, hBoxTR);
            //
            return MochiKit.DOM.TABLE(
                this.attrs, null, null, tableBody);
        }
        else
        {
            //
            var tableBody =  MochiKit.DOM.TBODY();
            //
            return MochiKit.DOM.TABLE(
                this.attrs, null, null, tableBody);
        }
    }
});

/**
 *
 */
BewypeJS.Ui.VBoxParser =
{
    parse: function (elementId)
    {
        // use parent parser
        return BewypeJS.Ui.BoxParser.parse("vertical", elementId);
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.BoxParser.swap(elementId, widget);
    }
};

/**
 *
 */
function updateVBoxTRList(classBox, boxItem)
{
    // init
    var styleStr = "";

    // init style str
    if (MochiKit.Base.isUndefinedOrNull(boxItem.padding))
    {
        // do nothing
        MochiKit.Base.noop();
    }
    else
    {
        styleStr += "padding: %s;" % boxItem.padding;
    }

    // update style fill
    if (!MochiKit.Base.isUndefinedOrNull(boxItem.fill)
        && boxItem.fill == true)
    {
        styleStr += " width: 100%;";
    }
    else
    {
        MochiKit.Base.noop();
    }

    //
    var element = null;
    //
    if (boxItem.widget instanceof BewypeJS.Ui.Box)
    {
        var params = {"tableWidth": "100%"};
        element = boxItem.widget.getDocFactory(params);
    }
    else
    {
        element = boxItem.widget.getDocFactory();
    }

    // set div for element
    var divElement = MochiKit.DOM.DIV(
                {
                    "class": classBox,
                    "style": styleStr
                },
                element
            );
    //
    var _attrs = {"class": classBox};
    return MochiKit.DOM.TR(_attrs, MochiKit.DOM.TD(_attrs, divElement));
};

/**
 *
 */
BewypeJS.Ui.VBox = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Box,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);
    },

    getDocFactory: function (params)
    {
        // update attrs
        this.attrs["cellspacing"] = this.spacing;

        //
        if (MochiKit.Base.isUndefinedOrNull(params))
        {
            MochiKit.Base.noop();
        }
        else
        {
            //
            var tableWidth = BewypeJS.Utils.getVal(params, "tableWidth");
            //
            if (MochiKit.Base.isUndefinedOrNull(tableWidth))
            {
                MochiKit.Base.noop();
            }
            else
            {
                this.attrs["width"] = tableWidth;
            }
        }

        //
        if (this.widgets.length > 0)
        {
            // get current class
            var _class = BewypeJS.Utils.getVal(this.attrs, "class");
            //
            var _updateFunc = MochiKit.Base.partial(
                    updateVBoxTRList, _class)
            //
            var vBoxTRList = MochiKit.Base.map(
                    _updateFunc, this.widgets);
            //
            var tableBody =  MochiKit.DOM.TBODY(null, vBoxTRList);
            //
            return MochiKit.DOM.TABLE(
                this.attrs, null, null, tableBody);
        }
        else
        {
            //
            var tableBody =  MochiKit.DOM.TBODY();
            //
            return MochiKit.DOM.TABLE(
                this.attrs, null, null, tableBody);
        }
    }
});

