
/**
 *
 */
function getColContents(classTv, modelRow, cellLayout)
{
    // we need rendering here
    var _doHtmlRendering = true;

    // prepare function for col elements
    var _getContentFunc = MochiKit.Base.partial(
                BewypeJS.Ui.Utils.getCellContent,
                modelRow,
                _doHtmlRendering
            );

    var attrs = {
                    "class": classTv
                };

    // set div for element
    var divElement = MochiKit.DOM.DIV(
                attrs,
                MochiKit.Base.map(
                        _getContentFunc,
                        cellLayout.cellRendererList,
                        cellLayout.kargsList,
                        cellLayout.funcList
                    )
            );

    //
    return MochiKit.DOM.TD(attrs, divElement);
}

/**
 *
 */
function getTreeviewTr(treeView, modelRow)
{
    // get treeview class
    var _class = BewypeJS.Utils.getVal(treeView.attrs, "class");
    // new function to update a specific cell
    var _getCellFunc = MochiKit.Base.partial(
            getColContents, _class, modelRow);
    //
    var _attrs = { "class": _class };

    //
    return MochiKit.DOM.TR(
                _attrs,
                MochiKit.Base.map(_getCellFunc, treeView.columnList)
            );
}

function getTreeviewTh(classAttrs, headerVisble, column)
{
    // get current class
    var _class = BewypeJS.Utils.getVal(classAttrs, "class");

    if(headerVisble == true)
    {
        var _headerLabel = new BewypeJS.Ui.Label(
                            {
                                "class": _class,
                                "text": column.title
                            }
                        );
        return MochiKit.DOM.TH(
                classAttrs,
                _headerLabel.getDocFactory()
                );
    }
    else
    {
        return MochiKit.DOM.TH(classAttrs, null);
    }
}

/**
 *
 */
BewypeJS.Ui.TreeView = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Container,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);
        //
        this.model = params["model"];
        //
        this.columnList = new Array();
        // assume None is the defatult header value
        this.headersVisible = new Array();
        // init treview main vars
        this.selection = null;
        //
        this.hadjustment = null;
        this.vadjustment = null;
    },

    getDocFactory: function (params)
    {
        //
        if (MochiKit.Base.isUndefinedOrNull(params))
        {
            MochiKit.Base.noop();
        }
        else
        {
            //
            var tableWidth = params["tableWidth"];
            //
            if (MochiKit.Base.isUndefinedOrNull(tableWidth))
            {
                MochiKit.Base.noop();
            }
            else
            {
                this.attrs["width"] = tableWidth;
            }
        }

        //
        if (this.model.length > 0)
        {
            // get current class
            var _class = BewypeJS.Utils.getVal(this.attrs, "class");
            // set update func
            var _classAttrs = {"class": _class};

            // get header row
            var _getThFunc = MochiKit.Base.partial(getTreeviewTh, _classAttrs);
            var _thRowList = MochiKit.Base.map(
                    _getThFunc, this.headersVisible, this.columnList);    
            // init thead row list
            var _thRow = MochiKit.DOM.TR(_classAttrs, _thRowList);
            var _tableHead = MochiKit.DOM.THEAD(_classAttrs, _thRow);
            // get other rows
            var _getTrFunc = MochiKit.Base.partial(getTreeviewTr, this);
            var _trList = [MochiKit.Base.map(_getTrFunc, this.model)];
            // set table body and update tr list
            var _tableBody = MochiKit.DOM.TBODY(_classAttrs, _trList);
            // do render
            return MochiKit.DOM.TABLE(this.attrs, _tableHead, null, _tableBody);
        }
        else
        {
            //
            var tableBody =  MochiKit.DOM.TBODY();
            //
            return MochiKit.DOM.TABLE(
                this.attrs, null, null, tableBody);
        }
    },

    getModel: function (params)
    {
        return this.model;
    },

    setModel: function (model)
    {
        this.model = model;
    },

    getSelection: function ()
    {
        return this.selection;
    },

    setSelection: function (selection)
    {
        this.selection = selection;
    },

    getHadjustment: function ()
    {
        return this.hadjustment;
    },

    setHadjustment: function (adjustment)
    {
        this.hadjustment = adjustment;
    },

    getVadjustment: function ()
    {
        return this.vadjustment;
    },

    setVadjustment: function (adjustment)
    {
        this.vadjustment = adjustment;
    },

    getHeadersVisible: function ()
    {
        return this.headersVisible;
    },

    setHeadersVisible: function (headersVisible)
    {
        if(headersVisible.length == this.headersVisible.length)
        {
            this.headersVisible = headersVisible;
        }
        else
        {
            MochiKit.Base.noop();
        }
    },

    appendColumn: function (column)
    {
        //
        this.columnList.push(column);
        this.headersVisible.push(true);
    },

    insertColumn: function (column, position)
    {
        //
        BewypeJS.Utils.insertToArray(this.columnList, column, position);
        BewypeJS.Utils.insertToArray(this.headersVisible, true, position);
    },

    removeColumn: function (column)
    {
        // remove column item and retrieve position in the list
        var _position = BewypeJS.Utils.removeFromArray(
                this.columnList, column);
        // remove header at previous position
        BewypeJS.Utils.removeFromArrayAtPosition(
                this.headersVisible, _position);
    }
});

