
/**
 *
 */
BewypeJS.Utils.Box =
{

    /**
     * Simple select flag update, useful for mapping
     */
    _setSelectedValue: function (activeList, optionElmt)
    {
        if(MochiKit.Base.findValue(activeList, optionElmt.value) == -1)
        {
            optionElmt.selected = null;
        }
        else
        {
            optionElmt.selected = "selected";
        }
    },
    
    setComboBoxActive: function (elementIdStr, activeList)
    {
        // get warning element
        var _selectElmt = MochiKit.DOM.getElement(elementIdStr);

        if (MochiKit.Base.isUndefinedOrNull(_selectElmt)
            || MochiKit.Base.isUndefinedOrNull(activeList))
        {
            // do nothing
            MochiKit.Base.noop();
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Utils.Box] setComboBoxActive - not found: "
                    + elementIdStr
                    );
            // DEBUG
        }
        else
        {
            // manage div, a, innerHTML values
            if (_selectElmt.tagName == "select"
                || _selectElmt.tagName == "SELECT")
            {
                // first unselect all
                var _updateFunc = MochiKit.Base.partial(
                        BewypeJS.Utils.Box._setSelectedValue, activeList);
                // do map for update
                MochiKit.Base.map(_updateFunc, _selectElmt.options);
            }
            else
            {
                // DEBUG
                MochiKit.Logging.logDebug(
                        "[Utils.Box] setComboBoxActive - invalid tagName "
                        + _selectElmt.tagName
                        + " for element : "
                        + elementIdStr
                        );
                // DEBUG
            }
        }
    },

    /**
     *
     */
    _getSelectedValue: function (activeList, optionElmt)
    {
        if(optionElmt.selected == true
            || optionElmt.selected == "selected")
        {
            activeList.push(optionElmt.value);
        }
        else
        {
            return null;
        }
    },
    
    getComboBoxActive: function (elementIdStr)
    {
        // get warning element
        var _selectElmt = MochiKit.DOM.getElement(elementIdStr);
        //
        if (MochiKit.Base.isUndefinedOrNull(_selectElmt))
        {
            // do nothing
            MochiKit.Base.noop();

            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Utils.Box] getComboBoxActive - not found: "
                    + elementIdStr
                    );
            // DEBUG
            // return result list
            return [[], 0];
        }
        else
        {
            // manage div, a, innerHTML values
            if (_selectElmt.tagName == "select"
                || _selectElmt.tagName == "SELECT")
            {
                // init result list
                var _activeList = [];
                // prepare update func
                _getSelectedValueFunc = MochiKit.Base.partial(
                        BewypeJS.Utils.Box._getSelectedValue,
                        _activeList)
                // do map for update
                 MochiKit.Base.map(
                        _getSelectedValueFunc,
                        _selectElmt.options
                        );
                 // return result list
                 return [_activeList, _selectElmt.options.length];
            }
            else
            {
                // DEBUG
                MochiKit.Logging.logDebug(
                        "[Utils.Box] getComboBoxActive - invalid tagName "
                        + _selectElmt.tagName
                        + " for element : "
                        + elementIdStr
                        );
                // DEBUG
                // return result list
                return [[], 0];
            }
        }
    },

    _parseCombo: function (modelCombo, optionElement)
    {
        //
        _modelRow = [
            optionElement.innerText,
            optionElement.value
            ];
        // update model
        modelCombo.push(_modelRow);
    },
                  
    parseComboBox: function (elementId)
    {
        // parse the box element
        var _comboBox = BewypeJS.Ui.ComboBoxParser.parse(elementId);
        //
        if(MochiKit.Base.isUndefinedOrNull(_comboBox))
        {
            return null;
        }
        else
        {
            // get current html element
            var _currentBox = MochiKit.DOM.getElement(elementId);
            // default rendererfor parsing
            var _cell = new BewypeJS.Ui.CellRendererText(
                        {
                            "class": _currentBox.className
                        }
                    );
            // add renderer
            _comboBox.packStart(_cell);
            // default text col for parsing
            _comboBox.addAttribute(_cell, "text", 0);
            // init model
            var _modelCombo = [];
            // prepare update model func
            var _parseFunc = MochiKit.Base.partial(
                    BewypeJS.Utils.Box._parseCombo, _modelCombo);
            // update model
            MochiKit.Base.map(_parseFunc, _currentBox.options);
            // update model
            _comboBox.setModel(_modelCombo);
            // return the combo widget
            return _comboBox;
        }
    },
                  
    updateComboBox: function (elementId, textColumn, modelCombo)
    {
        // parse the box element
        var _comboBox = BewypeJS.Ui.ComboBoxParser.parse(elementId);
        //
        if(MochiKit.Base.isUndefinedOrNull(_comboBox))
        {
            return null;
        }
        else
        {
            // get current html element
            var _currentBox = MochiKit.DOM.getElement(elementId);
            // default rendererfor parsing
            var _cell = new BewypeJS.Ui.CellRendererText(
                        {
                            "class": _currentBox.className
                        }
                    );
            // add renderer
            _comboBox.packStart(_cell);
            // default text col for parsing
            _comboBox.addAttribute(_cell, "text", textColumn);
            // update model
            _comboBox.setModel(modelCombo);
            // do swap at the end
            BewypeJS.Ui.ComboBoxParser.swap(elementId, _comboBox);
        }
    },

    /**
     * Simple checked flag update, useful for mapping
     */
    _setCheckedValue: function (activeList, buttonElmt)
    {
        var _value = null;
        if(buttonElmt.value == "true")
        {
            _value = true;
        }
        else if(buttonElmt.value == "false")
        {
            _value = false;
        }
        else if(buttonElmt.value == "null")
        {
            _value = null;
        }
        else
        {
            _value = buttonElmt.value;
        }

        if(MochiKit.Base.findValue(activeList, _value) == -1)
        {
            buttonElmt.checked = null;
        }
        else
        {
            buttonElmt.checked = "checked";
        }
    },
    
    setButtonBoxActive: function (elementIdStr, activeList)
    {
        // get warning element
        var _buttonBoxElmt = MochiKit.DOM.getElement(elementIdStr);

        if (MochiKit.Base.isUndefinedOrNull(_buttonBoxElmt)
            || MochiKit.Base.isUndefinedOrNull(activeList))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Utils.Box] setButtonBoxActive - not found: "
                    + elementIdStr
                    );
            // DEBUG
        }
        else
        {
            _buttonElmts = MochiKit.DOM.getElementsByTagAndClassName(
                    "INPUT", _buttonBoxElmt.className, _buttonBoxElmt
                    );
            // first unselect all
            var _updateFunc = MochiKit.Base.partial(
                    BewypeJS.Utils.Box._setCheckedValue, activeList
                    );
            // do map for update
            MochiKit.Base.map(_updateFunc, _buttonElmts);
        }
    },

    /**
     *
     */
    _getCheckedValue: function (activeList, buttonElmt)
    {
        if(buttonElmt.checked == true)
        {
            activeList.push(buttonElmt.value);
        }
        else
        {
            return null;
        }
    },
    
    getButtonBoxActive: function (elementIdStr)
    {
        // get warning element
        var _buttonBoxElmt = MochiKit.DOM.getElement(elementIdStr);

        if (MochiKit.Base.isUndefinedOrNull(_buttonBoxElmt))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Utils.Box] getButtonBoxActive - not found: "
                    + elementIdStr
                    );
            // DEBUG
            // return result list
            return [[], 0];
        }
        else
        {
            // get buttons of the group
            _buttonElmts = MochiKit.DOM.getElementsByTagAndClassName(
                    "INPUT", _buttonBoxElmt.className, _buttonBoxElmt
                    );
            // init result list
            var _activeList = [];
            // prepare update func
            _getSelectedValueFunc = MochiKit.Base.partial(
                    BewypeJS.Utils.Box._getCheckedValue,
                    _activeList)
            // do map for update
             MochiKit.Base.map(
                    _getSelectedValueFunc,
                    _buttonElmts
                    );
             // return result list
             return [_activeList, _buttonElmts.length];
        }
    },

    _parseButton: function (buttonBox, tdElement)
    {
        // source
        var _inputElmt = MochiKit.DOM.getFirstElementByTagAndClassName(
                    "INPUT", tdElement.className, tdElement);
        // 
        if(_inputElmt.type == "submit")
        {
            // parse & add button
            buttonBox.add(BewypeJS.Ui.ButtonParser.parse(_inputElmt));
        }
        else if(_inputElmt.type == "checkbox")
        {
            // parse check
            buttonBox.add(BewypeJS.Ui.CheckButtonParser.parse(_inputElmt));
        }
        else if(_inputElmt.type == "radio")
        {
            // parse radio
            buttonBox.add(BewypeJS.Ui.RadioButtonParser.parse(_inputElmt));
        }
        else
        {
            MochiKit.Base.noop();
        }
    },
                  
    parseButtonBox: function (boxType, elementId)
    {
        // parse the box element
        var _buttonBox = BewypeJS.Ui.ButtonBoxParser.parse(boxType, elementId);
        //
        if(MochiKit.Base.isUndefinedOrNull(_buttonBox))
        {
            return null;
        }
        else
        {
            // get element
            var _currentBox = MochiKit.DOM.getElement(elementId);
            // prepare add button func
            var _parseFunc = MochiKit.Base.partial(
                    BewypeJS.Utils.Box._parseButton, _buttonBox);
            // get td child
            var _boxElmnts = MochiKit.DOM.getElementsByTagAndClassName(
                    "TD", _currentBox.className, _currentBox);
            // append buttons
            MochiKit.Base.map(_parseFunc, _boxElmnts);
            //
            return _buttonBox;
        }
    },

    _addButton: function (buttonBox, buttonType, buttonGroup, buttonKey, modelRow)
    {
        // ensure class
        var _currentClass = BewypeJS.Utils.getVal(modelRow, "class");
        if(MochiKit.Base.isUndefinedOrNull(_currentClass))
        {
            modelRow["class"] = BewypeJS.Utils.getVal(buttonBox.attrs, "class");
        }
        else
        {
            MochiKit.Base.noop();
        }
        //
        var _button = null;
        // init button
        if(buttonType == "submit")
        {
            //
            _button = new BewypeJS.Ui.Button(modelRow);
        }
        else if(buttonType == "checkbox")
        {
            //
            _button = new BewypeJS.Ui.CheckButton(modelRow);
        }
        else if(buttonType == "radiobox")
        {
            //
            _button = new BewypeJS.Ui.RadioButton(modelRow);
            _button.setGroup(buttonGroup);
            _button.setName(buttonKey);
            // overide group
            buttonGroup = _button.getGroup();
        }
        else
        {
            // no button here!
            return;
        }

        //
        buttonBox.add(_button);
        //
        return _button;
    },
                  
    updateButtonBox: function (boxType, elementId, buttonType, buttonModel)
    {
        // parse the box element
        var _buttonBox = BewypeJS.Ui.ButtonBoxParser.parse(boxType, elementId);
        //
        if(MochiKit.Base.isUndefinedOrNull(_buttonBox))
        {
            return null;
        }
        else
        {
            // manage button group for radio box
            var _buttonGroup = null;
            // generate a random key for radio button management
            var _buttonKey = BewypeJS.Utils.generateKey(32);
            // prepare add button func
            var _addFunc = MochiKit.Base.partial(
                    BewypeJS.Utils.Box._addButton,
                    _buttonBox,
                    buttonType,
                    _buttonGroup,
                    _buttonKey
                    );
            // append buttons
            MochiKit.Base.map(_addFunc, buttonModel);
            // do swap at the end
            BewypeJS.Ui.ButtonBoxParser.swap(elementId, _buttonBox);
        }
    }
}

