
/**
 *
 */
BewypeJS.Utils =
{

    /**
     * Return a random value of a list.
     */
    getRandomValue: function (valueList)
    {
        return valueList[Math.floor(Math.random() * (valueList.length -1))];
    },

    /**
     * Return a randomly generated key according the length passed in parameter.
     */
    generateKey: function (keyLength)
    {
        // get upper chars
        var _upperChars = MochiKit.Base.map(
                String.fromCharCode, MochiKit.Iter.range(65, 91));
        // get lower chars
        var _lowerChars = MochiKit.Base.map(
                String.fromCharCode, MochiKit.Iter.range(97, 123));
        // concat all
        var _chars = [].concat(_upperChars, _lowerChars);

        // prepare get random char func
        _getRandomCharFunc = MochiKit.Base.partial(
                BewypeJS.Utils.getRandomValue,
                _chars)

        // get generate char list
        var _keyList = MochiKit.Base.map(
                _getRandomCharFunc,
                MochiKit.Iter.range(keyLength)
                );

        // return joined result
        return _keyList.join("");
    },

    /**
     *
     */
    strReplace: function (strText, srcPattern, dstPattern)
    {
        if(MochiKit.Base.isUndefinedOrNull(strText)
            || MochiKit.Base.isUndefinedOrNull(srcPattern)
            || MochiKit.Base.isUndefinedOrNull(dstPattern))
        {
            return strText;
        }
        else
        {
            var _splittedWords = strText.split(srcPattern)
            return _splittedWords.join(dstPattern);
        }
    },

    /**
     *
     * param wordsStr: ex.: test_word -> TestWord
     * type wordsStr: str
     */
    getClassName: function (wordsStr, pattern)
    {
        // ensure a pattern
        var _patternForSplit = null;
        // set default pattern
        if(MochiKit.Base.isUndefinedOrNull(pattern))
        {
            _patternForSplit = "_";
        }
        else
        {
            _patternForSplit = pattern;
        }

        // do split
        if(MochiKit.Base.isUndefinedOrNull(wordsStr))
        {
            return null;
        }
        else
        {
            // prepare string to be camelized
            var _strToCamelize = BewypeJS.Utils.strReplace(
                    wordsStr,
                    _patternForSplit,
                    "-");
            // return class name
            return MochiKit.Base.camelize("-" + _strToCamelize);
        }
    },

    /**
     * Simple method for mapping.
     */
    _setName: function(name, widget)
    {
        widget.name = name;
    },
           
    /**
     * Only set val if not null.
     */
    setVal: function (dict, key, val)
    {
        //
        if (MochiKit.Base.isUndefinedOrNull(val))
        {
            //
            MochiKit.Base.noop();
        }
        else
        {
            dict[key] = val;
        }
    },

    /**
     * Ensure we get value from a dict or return null.
     * Avoid error risk!
     */
    getVal: function (dict, key)
    {
        if(MochiKit.Base.isUndefinedOrNull(dict)
            || MochiKit.Base.findValue(MochiKit.Base.keys(dict), key) == -1)
        {
            // key not found in the dict or no dict
            return null;
        }
        else
        {
            // should be ok now
            return dict[key];
        }
    },

    /**
     *
     */
    getAttrVal: function (element, key)
    {
        var _hrefAttr = element.getAttributeNode(key);
        if(MochiKit.Base.isUndefinedOrNull(_hrefAttr))
        {
            return null;
        }
        else
        {
            return _hrefAttr.value;
        }

    },

    /**
     *
     */
    getFieldValue: function(elementId)
    {
        // get warning element
        var _fieldElmt = MochiKit.DOM.getElement(elementId);
        //
        if (MochiKit.Base.isUndefinedOrNull(_fieldElmt))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[JSUtils] getFieldValue - not found: " + elementId);
            // DEBUG
            // explicitly return null value
            return null;
        }
        else
        {
            // manage div, a, innerHTML values
            if (_fieldElmt.tagName.toLowerCase() == "input"
                || _fieldElmt.tagName.toLowerCase() == "textarea")
            {
                return MochiKit.Format.strip(_fieldElmt.value);
            }
            else if (_fieldElmt.tagName.toLowerCase() == "img")
            {
                return MochiKit.Format.strip(_fieldElmt.src);
            }
            else
            {
                return MochiKit.Format.strip(_fieldElmt.innerText);
            }
        }
    },

    /**
     * Simple form field updater updater.
     *
     * Find the field and update the field value.
     */
    updateField: function (elementId, value, defaultValue)
    {
        // ensure valid value
        if (MochiKit.Base.isUndefinedOrNull(value)
            && MochiKit.Base.isUndefinedOrNull(defaultValue))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[JSUtils] updateField - null value for: " + elementId);
            // DEBUG
            value = "";
        }
        else if (MochiKit.Base.isUndefinedOrNull(value))
        {
            value = defaultValue;
        }
        else
        {
            // ok
            MochiKit.Base.noop();
        }
        // get warning element
        var _fieldElmt = MochiKit.DOM.getElement(elementId);
        //
        if (MochiKit.Base.isUndefinedOrNull(_fieldElmt))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[JSUtils] updateField - not found: " + elementId);
            // DEBUG
        }
        else
        {
            // manage div, a, innerHTML values
            if (_fieldElmt.tagName.toLowerCase() == "input"
                || _fieldElmt.tagName.toLowerCase() == "textarea")
            {
                _fieldElmt.value = value;
            }
            else if (_fieldElmt.tagName.toLowerCase() == "img")
            {
                _fieldElmt.src = value;
            }
            else if (_fieldElmt.tagName.toLowerCase() == "div")
            {
                _fieldElmt.innerHTML = value;
            }
            else
            {
                _fieldElmt.innerText = value;
            }
        }
    },
    
    /**
     * Build the commonly used secure url for rpc connection.
     */
    getRpcUrl: function (actionName)
    {
        // get elements info
        var _rpcUrlElmt = MochiKit.DOM.getElement("RpcUrl");

        if(MochiKit.Base.isUndefinedOrNull(_rpcUrlElmt)
            || MochiKit.Base.isUndefinedOrNull(actionName)
            || actionName == "")
        {
            MochiKit.Base.noop();
        }
        else
        {
            // just for that :s
            return _rpcUrlElmt.value + "/" + actionName;
        }
    }
}

